#include <gynlibs.cpp>
#include <math.h>
#include <windows.h>

uint32_t FreqMap[256][256];
uint8_t BitMap[256][256];


LONG_MAIN(argc, argv)
{
  CHECK_ARGS(2, "usage: file2d <filename>");

  size_t size;
  uint8_t *data = FileGetContent(argv[1], &size);
  CHECK_FILEGET_DIE(argv[1], data);

  uint32_t max = 0;
  size_t i, j;

  // Mark frequencies
  for(i = 0; i < size - 1; i++)
  {
    uint8_t x = data[i];
    uint8_t y = data[i+1];

    FreqMap[y][x]++;

    if(FreqMap[y][x] > max)
      max = FreqMap[y][x];
  }

  float lmax = log((float)max);

  printf("%i %f\n", max, lmax);

  // Fix frequencies
  for(j = 0; j < 256; j++)
  {
    for(i = 0; i < 256; i++)
    {
      if(FreqMap[j][i] > 0)
      {
        float fq = log((float)FreqMap[j][i]);
        BitMap[j][i] = (uint8_t)((fq / lmax) * 255.0f);
        //printf("FreqMap %i %f (%i %f) -> %u\n", FreqMap[j][i], fq, max, lmax, BitMap[j][i]);
      }
      else
      {
        BitMap[j][i] = 0;
      }






      //FreqMap[j][i] = (uint8_t)(((float)FreqMap[j][i] / (float)max) * 255.0f);
    }
  }

  // Save bmp
  BITMAPFILEHEADER fh = {
    0x4D42,
    sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * 256 + 256*256,
    0,
    0,
    sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * 256
  };

  BITMAPINFOHEADER ih = {
    sizeof(BITMAPINFOHEADER),
    256,
    256,
    1,
    8,
    BI_RGB,
    0,
    0,
    0,
    0,
    0
  };

  static RGBQUAD pal[256];
  for(i = 0; i < 256; i++)
  {
    pal[i].rgbRed   = 0xff - i;
    pal[i].rgbBlue  = 0xff - i;
    pal[i].rgbGreen = 0xff;
  }

  static char FileName[256];
  snprintf(FileName, sizeof(FileName) - 1, "%s.bmp", argv[1]);

  FILE *f = fopen(FileName, "wb");
  if(f)
  {
    fwrite(&fh, 1, sizeof(fh), f);
    fwrite(&ih, 1, sizeof(ih), f);
    fwrite(pal, 1, sizeof(pal), f);
    fwrite(BitMap, 1, sizeof(BitMap), f);
    fclose(f);
  }
  else
  {
    printf("could not create %s\n", FileName);
  }


  return 0;
}

