#include <gynlibs.cpp>
#include <math.h>
#include <windows.h>

#pragma pack(push, 1)
uint32_t FreqMap[256][256];
uint8_t BitMap[256][256][3];
uint32_t OutMap[256][256];
#pragma pack(pop)


LONG_MAIN(argc, argv)
{
  CHECK_ARGS(2, "usage: file2d <filename>");

  size_t size;
  uint8_t *data = FileGetContent(argv[1], &size);
  CHECK_FILEGET_DIE(argv[1], data);

  uint32_t max = 0;
  uint32_t min = 0xffffffff;
  size_t i, j;

  // Mark frequencies
  for(i = 0; i < size - 1; i++)
  {
    uint8_t x = data[i];
    uint8_t y = data[i+1];

    FreqMap[y][x]++;

    if(FreqMap[y][x] > max)
      max = FreqMap[y][x];
  }

  for(j = 0; j < 256; j++)
  {
    for(i = 0; i < 256; i++)
    {
      if(FreqMap[j][i] < min)
        min = FreqMap[j][i];
    }
  }

  float lmax = log((float)max);
  float lmin = log((float)min);

  if(min == 0)
    lmin = 0;

  printf("%i %f\n", max, lmax);
  static RGBQUAD pal[1024];

  // Black -> Blue
  // Blue -> Green
  // Green -> Red
  // Red -> Yellow
  
//  RGBQUAD dst = { 17, 51, 96 };
  RGBQUAD dst = { 18, 74, 115 };

  float step_to_zero[3];
  step_to_zero[0] = (0.0f - (float)dst.rgbBlue) / 256.0f;
  step_to_zero[1] = (0.0f - (float)dst.rgbGreen) / 256.0f;
  step_to_zero[2] = (0.0f - (float)dst.rgbRed) / 256.0f;

  float step_from_white[3];
  step_from_white[0] = ((float)dst.rgbBlue - 255.0f) / 256.0f;
  step_from_white[1] = ((float)dst.rgbGreen - 255.0f) / 256.0f;
  step_from_white[2] = ((float)dst.rgbRed - 255.0f) / 256.0f;



  for(i = 0; i < 256; i++)
  {
    pal[i].rgbBlue  = 0xff + step_from_white[0] * i;
    pal[i].rgbGreen = 0xff + step_from_white[1] * i;
    pal[i].rgbRed   = 0xff + step_from_white[2] * i;

    pal[i+256*1].rgbRed   = dst.rgbRed   + step_to_zero[2] * i;
    pal[i+256*1].rgbGreen = dst.rgbGreen + step_to_zero[1] * i;
    pal[i+256*1].rgbBlue  = dst.rgbBlue  + step_to_zero[0] * i;

/*    pal[i+64*1].rgbRed   = i*4;
    pal[i+64*1].rgbGreen = 0xff;
    pal[i+64*1].rgbBlue  = 0;

    pal[i+64*2].rgbRed   = 0xff;
    pal[i+64*2].rgbGreen = 0xff-i*4;
    pal[i+64*2].rgbBlue  = 0;

    pal[i+64*3].rgbRed   = 0xff-i*4;
    pal[i+64*3].rgbGreen = 0;
    pal[i+64*3].rgbBlue  = 0;*/
  }




  // Fix frequencies
  for(j = 0; j < 256; j++)
  {
    for(i = 0; i < 256; i++)
    {
      if(FreqMap[j][i] > 0)
      {
        float fq = log((float)FreqMap[j][i]);
        float final = (fq - lmin) / (lmax - lmin);
        OutMap[j][i] = (uint32_t)(final * 511.0f);
        //OutMap[j][i] = (uint32_t)(((float)FreqMap[j][i] / float(max)) * 1023.0f);
       //printf("FreqMap %i %f (%i %f) -> %u\n", FreqMap[j][i], fq, max, lmax, OutMap[j][i]);
      }
      else
      {
        OutMap[j][i] = 0;
      }

      BitMap[255-j][i][0] = pal[OutMap[j][i]].rgbBlue;
      BitMap[255-j][i][1] = pal[OutMap[j][i]].rgbGreen;
      BitMap[255-j][i][2] = pal[OutMap[j][i]].rgbRed;
      //FreqMap[j][i] = (uint8_t)(((float)FreqMap[j][i] / (float)max) * 255.0f);
    }
  }

  // Save bmp
  BITMAPFILEHEADER fh = {
    0x4D42,
    sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + 256*256 * 3,
    0,
    0,
    sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER)
  };

  BITMAPINFOHEADER ih = {
    sizeof(BITMAPINFOHEADER),
    256,
    256,
    1,
    24,
    BI_RGB,
    0,
    0,
    0,
    0,
    0
  };



  static char FileName[256];
  snprintf(FileName, sizeof(FileName) - 1, "%s.bmp", argv[1]);

  FILE *f = fopen(FileName, "wb");
  if(f)
  {
    fwrite(&fh, 1, sizeof(fh), f);
    fwrite(&ih, 1, sizeof(ih), f);
    //fwrite(pal, 1, sizeof(pal), f);
    fwrite(BitMap, 1, sizeof(BitMap), f);
    fclose(f);
  }
  else
  {
    printf("could not create %s\n", FileName);
  }


  return 0;
}

