#include "gynlibs.h"

unsigned char *
FileGetContent(const char *FileName, size_t *Size)
{
  FILE *f;
  size_t FileSize;
  unsigned char *Data;

  // Open the file
  f = fopen(FileName, "rb");
  if(!f)
    return NULL;

  // Get file size
  fseek(f, 0, SEEK_END);
  FileSize = ftell(f);
  fseek(f, 0, SEEK_SET);

  // Allocate memory
  Data = (unsigned char*)malloc(FileSize + 1);
  if(!Data)
  {
    fclose(f);
    return NULL;
  }

  // Read file content
  FileSize = fread(Data, 1, FileSize, f);
  Data[FileSize] = 0;

  // Close the file
  fclose(f);

  // Return
  if(Size) // Size is optional
    *Size = FileSize;
  return Data;
}

unsigned char *
FileGetContentAsText(const char *FileName, size_t *Size)
{
  FILE *f;
  size_t FileSize;
  unsigned char *Data;

  // Open the file
  f = fopen(FileName, "r");
  if(!f)
    return NULL;

  // Get file size
  fseek(f, 0, SEEK_END);
  FileSize = ftell(f);
  fseek(f, 0, SEEK_SET);

  // Allocate memory
  Data = (unsigned char*)malloc(FileSize + 1);
  if(!Data)
  {
    fclose(f);
    return NULL;
  }

  // Read file content
  FileSize = fread(Data, 1, FileSize, f);
  Data[FileSize] = 0;

  // Close the file
  fclose(f);

  // Return
  if(Size) // Size is optional
    *Size = FileSize;
  return Data;
}

bool
FileSetContent(const char *FileName, void *Data, size_t Size)
{
  FILE *f;
  size_t WritenSize;
  bool State = true;

  // Open the file
  f = fopen(FileName, "wb");
  if(!f)
    return false;

  // Write
  WritenSize = fwrite(Data, 1, Size, f);
  if(WritenSize != Size)
    State = false;

  // Close the file
  fclose(f);

  return State;
}

bool
IsAlpha(int c)
{
  return ((c >= 'a' && c <= 'z') ||
          (c >= 'A' && c <= 'Z'));
}

bool
IsNum(int c)
{
  return (c >= '0' && c <='9');
}

bool
IsAlphaNum(int c)
{
  return (IsAlpha(c) || IsNum(c));
}

bool
IsHex(int c)
{
  return (IsAlpha(c) ||
          (c >= 'a' && c <= 'f') ||
          (c >= 'A' && c <= 'F'));       
}

bool
IsPrintable(int c)
{
  return (c >= 0x20 && c <= 0x7e);
}

bool
IsWhiteChar(int c)
{
  return (c == ' ' || c == '\t' || c == '\r' || c == '\n');
}

int *
MakeHistogram(void *Data, size_t Size)
{
  int *Array;
  size_t i;
  unsigned char *Bytes = (unsigned char*)Data;

  // Alloc
  Array = (int*)malloc(256 * sizeof(int));
  if(!Array)
    return NULL;

  // Clean
  memset(Array, 0, 256 * sizeof(int));

  // Make
  for(i = 0; i < Size; i++)
    Array[Bytes[i]]++;

  // Return
  return Array;
}

float
Entropy(void *Data, size_t Size)
{
  int *Histogram;
  float En = 0.0f;
  float InvSize = 1.0f / (float)Size;
  int i;

  Histogram = MakeHistogram(Data, Size);
  if(!Histogram)
    return -1.0f;

  for(i = 0; i < 256; i++)
    En += Histogram[i] * Histogram[i] * InvSize;

  En = 1.0f - En * InvSize;

  return En;
}

